@@ -44,17 +44,21 @@ import six
 import pickle
 import pprint
 import sys
+import difflib
 
 __all__ = [
     "freeze",
     "freeze_fast",
     "freeze_stable",
-    "frozen_equal_assert",
+    "tree_diff_assert",
     "stable_hash",
     "recursive_hash",
+    "tree_diff",
     "flatten",
+    "frozen_equal_assert",
     "pickle_protocol",
-    "vformat"
+    "vformat",
+    "transparent_repr"
 ]
 
 
@@ -74,13 +78,20 @@ try:
 except:
     _string_types = (six.string_types)
 
+_builtin_function_or_method_type = type(dict().get)
 
-def freeze(data_structure):
+
+# At the moment the freeze variants don't share code. My merged version of
+# freeze was quite slow. Unit I come up with a better solution all freeze
+# variants have to be maintained.
+def freeze(data_structure, stringify=False):
     """Freezing a data-structure makes all items tuples and therefore
     all levels are comparable/hashable/sortable. It has cycle detection
     and will freeze any kind of python object.
 
     :param   data_structure: The structure to convert
+    :param        stringify: Stringify all leaves
+    :type         stringify: bool
 
     >>> freeze([
     ...     'a',
@@ -108,7 +119,12 @@ def freeze(data_structure):
     >>> freeze(testdata) == freeze_fast(testdata)
     True
 
-    >>> len(freeze(sys)) > 0
+    Hash anything!
+
+    >>> a = freeze(sys)
+    >>> len(a) > 0
+    True
+    >>> hash(a) != None
     True
     """
 
@@ -153,12 +169,20 @@ def freeze(data_structure):
                 pass
             if tlen != -1:
                 return tuple([freeze_helper(x) for x in data_structure])
+        # To guarantee that the result is hashable we do not return
+        # builtin_function_or_method
+        if stringify or isinstance(
+            data_structure,
+            _builtin_function_or_method_type
+        ):
+            return str(data_structure)
         return data_structure
     return freeze_helper(data_structure)
 
 
 def freeze_fast(data_structure):
-    """Like freeze but has no cycle detection.
+    """Like freeze but has no cycle detection and it doesn't
+    guarantee that the result is hashable.
 
     :param   data_structure: The structure to convert
 
@@ -206,7 +230,38 @@ def freeze_fast(data_structure):
     return freeze_fast_helper(data_structure)
 
 
-def freeze_stable(data_structure, assume_key=False):
+def _recursive_sort(data_structure, assume_key=False):
+    # We don't sort strings
+    if not isinstance(data_structure, _string_types):
+        tlen = -1
+        # If item has a length we frezze it
+        try:
+            tlen = len(data_structure)
+        except:
+            pass
+        if tlen != -1:
+            if assume_key and tlen == 2:
+                return tuple([
+                    _recursive_sort(
+                        x,
+                        assume_key=assume_key
+                    ) for x in data_structure
+                ])
+            else:
+                return tuple(sorted(
+                    [_recursive_sort(
+                        x,
+                        assume_key=assume_key
+                    ) for x in data_structure],
+                    key=lambda x: pickle.dumps(
+                        x,
+                        protocol=pickle_protocol
+                    )
+                ))
+    return data_structure
+
+
+def freeze_stable(data_structure, assume_key=False, stringify=False):
     """Like freeze but results are stable regarding sorting.
 
     The stable functionality changes the datestructure (sorting)!
@@ -219,10 +274,15 @@ def freeze_stable(data_structure, assume_key=False):
     creates smaller structures for flatten and a smaller
     difference when comparing two flattened structures.
 
+    stringify allow to freeze_stable anything.
+
     :param   data_structure: The structure to convert
-    :param   assume_key:     Assume that substructures of len() == 2
+    :param       assume_key: Assume that substructures of len() == 2
                              are key-value pairs -> don't sort
-    :type    assume_key:     bool
+    :type        assume_key: bool
+    :param        stringify: Stringify all leaves
+    :type         stringify: bool
+
 
     >>> a = freeze_stable([
     ...     'a',
@@ -249,9 +309,27 @@ def freeze_stable(data_structure, assume_key=False):
 
     >>> b"sdf" == freeze_stable({3: b"sdf", 4: 245234534})[1][1]
     True
+
+    >>> a = freeze_stable([
+    ...     'a',
+    ...     [3, 4],
+    ...     {'a': [3, {'w' : set([4, '3', frozenset([3,5,2])])}]},
+    ...     []
+    ... ], stringify = True)
+    >>> a
+    ((), ((((((('2', '3', '5'), '3', '4'), 'w'),), '3'), 'a'),), ('3', '4'), 'a')
+    >>> a = freeze_stable([
+    ...     'a',
+    ...     [5, 4],
+    ...     {'a': [3, {'w' : set([4, '3', frozenset([3,5,2])])}]},
+    ...     []
+    ... ], stringify = True, assume_key=True)
+    >>> a
+    ((), ('5', '4'), 'a', (('a', ('3', (('w', (('2', '3', '5'), '3', '4')),))),))
     """
 
-    identity_set = set()
+    if not stringify:
+        identity_set = set()
 
     def freeze_stable_helper(data_structure):
         # Cycle detection
@@ -303,8 +381,21 @@ def freeze_stable(data_structure, assume_key=False):
                             protocol=pickle_protocol
                         )
                     ))
+        # To guarantee that the result is hashable we do not return
+        # builtin_function_or_method
+        if isinstance(data_structure, _builtin_function_or_method_type):
+            return str(data_structure)
         return data_structure
-    return freeze_stable_helper(data_structure)
+    if stringify:
+        return _recursive_sort(
+            freeze(
+                data_structure,
+                stringify=True
+            ),
+            assume_key=assume_key
+        )
+    else:
+        return freeze_stable_helper(data_structure)
 
 
 def stable_hash(data_structure):
@@ -363,9 +454,9 @@ def recursive_hash(data_structure):
         except:
             pass
         if tlen != -1:
-            hashs = tuple(stable_hash(item) for item in data_structure)
-            return tuple(hashs)
-    return recursive_hash(data_structure)
+            hashs = tuple(recursive_hash(item) for item in data_structure)
+            return hash(tuple(hashs))
+    return hash(data_structure)
 
 
 def _flatten_helper(iterable, pathlist, path, parent_index=False):
@@ -426,8 +517,114 @@ def _flatten_helper(iterable, pathlist, path, parent_index=False):
             index += 1
 
 
+def tree_diff(a, b, n=5, deterministic=True):
+    """Freeze and stringify any data-structure or object, traverse
+    it depth-first in-order and apply a unified diff.
+
+    Depth-first in-order is just like structure would be printed.
+
+    Annotation:
+
+    "(           x"       Going down to level: x
+    ")           x"       Going one level up from: x
+
+    :param             a: data_structure a
+    :param             b: data_structure b
+    :param             n: lines of context
+    :type              n: int
+
+
+    >>> a = [
+    ...     'a',
+    ...     [3, 4],
+    ...     {'a': [3, {'w' : set([4, '3', frozenset([3,5,2])])}]},
+    ...     []
+    ... ]
+    >>> b = [
+    ...     'a',
+    ...     [4, 3],
+    ...     {'a': [3, {'w' : set([4, '3', frozenset([2,5,3])])}]},
+    ...     []
+    ... ]
+    >>> transparent_repr("\\n".join(tree_diff(a, b).split("\\n")[2:]))
+    @@ -1,8 +1,8 @@
+     ('a',
+    - ('3',
+    -  '4'),
+    + ('4',
+    +  '3'),
+      (('a',
+        ('3',
+         (('w',
+           (('2',
+             '3',
+
+    >>> a = [
+    ...     'a',
+    ...     [3, 4],
+    ...     {'a': [3, {'w' : set([4, 'tree', frozenset([3,5,2])])}]},
+    ...     []
+    ... ]
+    >>> b = [
+    ...     'a',
+    ...     [4, 3],
+    ...     {'a': [3, {'w' : set([4, '3', frozenset([2,5,3])])}]},
+    ...     []
+    ... ]
+    >>> transparent_repr("\\n".join(tree_diff(a, b).split("\\n")[2:]))
+    @@ -1,12 +1,12 @@
+     ('a',
+    - ('3',
+    -  '4'),
+    + ('4',
+    +  '3'),
+      (('a',
+        ('3',
+         (('w',
+           (('2',
+             '3',
+             '5'),
+    -       'tree',
+    +       '3',
+            '4')),))),),
+      ())
+    """
+    a = freeze(a, stringify=True)
+    b = freeze(b, stringify=True)
+    if deterministic:
+        a = vformat(a).split("\n")
+        b = vformat(b).split("\n")
+    else:
+        a = vformat(_recursive_sort(a)).split("\n")
+        b = vformat(_recursive_sort(b)).split("\n")
+    return "\n".join(difflib.unified_diff(a, b, n=n, lineterm=""))
+
+
+def tree_diff_assert(a, b, n=5, deterministic=True):
+    """Assert if a equals b. Freeze and stringify any data-structure or object,
+    traverse it depth-first and apply a unified diff, to display the result.
+
+    :param             a: data_structure a
+    :param             b: data_structure b
+    :param             n: lines of context
+    :type              n: int
+    :param deterministic: Do not sort the tree
+    """
+    a = freeze(a, stringify=True)
+    b = freeze(b, stringify=True)
+    if deterministic:
+        a = vformat(a).split("\n")
+        b = vformat(b).split("\n")
+    else:
+        a = vformat(_recursive_sort(a)).split("\n")
+        b = vformat(_recursive_sort(b)).split("\n")
+    msg = "\n".join(difflib.unified_diff(a, b, n=n, lineterm=""))
+    if len(msg) != 0:
+        assert False, "difference: \n%s" % msg
+
+
 def flatten(data_structure, assume_key=True):
-    """Converts a data_structure to a flat set of paths.
+    """Converts a data-structure to a flat set of paths.
     It makes it easier to comapre data-structures in unittests.
 
     Creating a set from a flattened structure can be lossy. This is the
@@ -543,13 +740,22 @@ class TransparentRepr(object):
         else:
             return super(TransparentRepr, self).__repr__()
 
+    def split(self, *args, **kwargs):
+        return self.repr_str.split(*args, **kwargs)
+
+
+def transparent_repr(string):
+    """The result is __repr__ transparent. Non-printable
+    characters won't be escaped"""
+    return TransparentRepr(string)
+
 
 def vformat(*args, **kwargs):
     """A pformat wrapper that produces narrow representations of
     data-structures. The result is __repr__ transparent. Non-printable
     characters won't be escaped"""
     return TransparentRepr(
-        pprint.pformat(*args, width=10, **kwargs)
+        pprint.pformat(*args, width=1, **kwargs)
     )
 
 
